#!/usr/local/bin/perl

# GTYPE2ALOHOMORA.pl
# Copyright(c) 2006 	Franz Rueschendorf

# converts multi-sample GTYPE outfile to ALOHOMORA-format
# assumes confidence rate column in infile
# Created 21-Sep-2006


use Cwd;
use strict;
use File::Copy;
use File::Basename;
use Getopt::Long;

###############################  V A R I A B L E S  ############################
my $work_dir=cwd();
my $datum = scalar localtime;
my $infile="";
my $files="";
my $skipfile="skipsamples.txt";
my %skip=();
my $th=0.30;


###################################  R U N  ####################################
if ($#ARGV < 0) { &Help;}

&GetArguments;
&ReadSkipList;

my @files=glob("$files");
foreach (@files) {
	$infile=$_;
	&ReadInfile;
}


########################  Read GTYPE multi-sample file  ########################
sub ReadInfile {
	print "Read infile   : $infile\n";
	
	if (! -f $infile) {
		print "\tno GTYPE file $infile\n";
		return;
	}
	
	my $basename=basename($infile);
	my @name=split('\.',$basename);
	my $outfile="$name[0]_${th}.txt";
	my %call=();
	my %conf=();
	my @samples=();
	my $pid="";
	my $c_snp="";
	my $switch=0;
	my $maxConf=0.0;
	my $ma=0;
	my $c=0;

	open (IN, "$infile") || die "could not open GTYPE file $infile :$!\n";
	open (OUT, "> $outfile") || print "could not open outfile $outfile :$!\n";
	print "Write outfile : $outfile\n";
	while (<IN>){
		next if /^$/;
		next if /^Dynamic/;
		next if /^MPAM/;
		
		if ($_ =~ /SNP ID/) {
			chomp;
			my @v=split('\t',$_);
			for (my $i=1;$i<=$#v;$i++) {
				if ($v[$i] eq "SNP ID") { $c_snp=$i;}
				elsif ($v[$i] =~ /_Call/) { 
					$pid=$v[$i];
					$pid =~ s/_Call//;
 					if ( ! exists $skip{$pid}) {		# exclude samples in list
#					if (  exists $skip{$pid}) {		# include samples in list
						$call{$pid}=$i;
						push (@samples,$pid);
					}
				}
				elsif ($v[$i] =~ /_Confidence/) { 
					$pid=$v[$i];
					$pid =~ s/_Confidence//;
 					if ( ! exists $skip{$pid}) {		# exclude samples in list
#					if (  exists $skip{$pid}) {		# include samples in list
						$conf{$pid}=$i;
					}
				}
				else {;}
			}
			print OUT "SNP ID";
			foreach (@samples) {
				print OUT "\t$v[$call{$_}]";
			}
			print OUT "\n";
			$switch=1;
			next;
		}			 
				

		if ($switch == 1) {
			chomp;
			my @v=split('\t',$_);
			if ( $v[$c_snp] =~ /^AFFX/) {next;}
			else { print OUT "$v[$c_snp]";}
			foreach my $pid (@samples) {
				if (exists $conf{$pid}) {
					if ( $v[$conf{$pid}] <= $th) { 
						print OUT "\t$v[$call{$pid}]"; 
					}
					else { print OUT "\tNC"; }
					if (( $v[$conf{$pid}] > $maxConf) && ( $v[$call{$pid}] ne "NoCall")) { $maxConf=$v[$conf{$pid}];}
				}
				else {print OUT "\t$v[$call{$pid}]"; }

			}
			print OUT "\n";
			$ma++;
			my $nr= $ma % 10000;
			if ($nr == 0. ) {
				print " ."; $c++;
				if ($c == 5) { print " "; }
				if ($c == 10) { print "\n"; $c=0; }
			}
		}
	}
	
	close (IN);
	close (OUT);
	
	my $sa= @samples;	
	print "\n\t$ma markers\t$sa samples\n\n";
	print "max Confidence with Call: ${maxConf}, threshold: $th\n";
	print "\nDone\n\n";

}

#########################   R E A D  S K I P  L I S T  #########################

sub ReadSkipList {

	# Skip samples from extra skiplist (e.g. doubled samples, bad callrate)

	if (-f $skipfile) {
		print "\nRead skiplist : $skipfile\n";
		open (IN, "$skipfile") || die "could not open $skipfile :$!\n";
		while (<IN>){
			next if /^$/;
			next if /^#/;
			chomp;
			my @row=split(' ',$_);
			$skip{$row[0]}=1;				# create hash of samples to skip
		}
		close (IN);
	}
	else {
		print "skiplist not selected\n";
	}	
}
###########################  Eval Command Line    ##############################

sub GetArguments {
	GetOptions(
		"f=s" => \$files,
		"c=s" => \$th,
		"help|?|h" => sub { &Help;}
	);
}

###################################  H E L P  ##################################
sub Help {
	print "\nusage: perl \[path\]\\gtype2alohomora.pl -c <confidence_threshold> -f <infile>\n";
	print "\n";
	exit 0;
}
###################################   E N D   ##################################
