#!/usr/bin/perl

#
# gh_snp_info.pl :  Prepare Genehunter Information Content for displaying with gnuplot
# Syntax    :  gh_snp_info.pl [-h] [-clean] [<infile>]
# Infiles   :  gh_<v>.out (default)
# Outfiles  :  myghinfo.gp, chr_line and INFO.plt
# Modified  : 22-Jan-2004	FR

use Cwd;

print "\n\n";
print " * * * * * * * * * * * GH_SNP_INFO.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * View Genehunter results with GNUPLOT  * * * * * * * \n";

my $nodelete =0;
my $hline=1;

if ($ARGV[0] =~ "-h" ) { eval Help;}
if ($ARGV[0] =~ "-v" ) { eval Help;}
if ($ARGV[0] =~ "-clean" ) { eval Clean;}
if ($ARGV[0] =~ "-nohline" ) {$hline=0;}

print "\nPlease wait ...\n";

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my @xsums=();
my $xsum = 0.0;
my $xmin = 0;

my $ymax = 1;
my $ymin = 0.;

my $pfad = getcwd();
$datum = scalar localtime;
my %chrlabel=();


#######################################    M A I N   #############################															
open (out4,">$pfad/INFO.plt") || die "could not open INFO.plt :$!\n";
open (OUT8,">$pfad/INFOdata.txt") || die "could not open INFOdata.txt :$!\n";

#foreach my $dir (c18,c19) { # test
foreach my $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,cx,c24,cxy) {
	if (-d $dir) {
		$gh_tsh=0;	# is there a total stat het for this chrom
		foreach my $i ( 1..30) {
			$ghout="gh_$i\.out";	
			$in_file = "$dir/$ghout";
			if (-f "$in_file") {	  
				print "File: $in_file"; 
				&ReadINFO(); 
				print "\n";
			}
		} 
		if ($gh_tsh==1) {		# chromosome label
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
		}
		push (@xsums,$xsum);
		print out4 "\n";  
	}
	else { print "$dir not existing \n";
	}
} # end foreach

close (out4);
close (OUT8);
&print_chr_line;
&pr_batch;

print "\nYou can now view results by \"gnuplot myghinfo.gp\" \n\n";
system ("gnuplot myghinfo.gp");

print "\nready ...\n\n";
if ($nodelete == 0) { &Clean;}

#################################   R E A D I N F O   #################################  

sub ReadINFO {
	my @v=();	# Array infile (Genehunter outfile)
	open (infile, "$in_file") || die "could not open $in_file :$!\n";
	while (<infile>){
		chomp;
		$_ =~ s/,-/, -/g;
		$_ =~ s/-INFINITY/-99.99/g;
		push (@v,$_);
	}
	close (infile);

	my $jetzt = $maxlines;
	my $xmax = 0.0;
	my $count=0;
	my $c_tsh=0;	# Counter total stat het - lines

	foreach $line (@v) {
		@feld =	split(' ',$line);
		$count++;
		if (($feld[3] =~ /het/) && ($feld[1] =~ /total/))   { 
			print " Read: total stat het";
			$c_tsh++;
			$gh_tsh=1;
		}
	
		if ($feld[2] =~ /alpha,/)   { $jetzt = $count;}
		if ($feld[0] =~ /file/) { $jetzt = $maxlines;}

		if (($feld[5] =~ /npl_plot.ps/) || ($#feld < 1)) { $jetzt = $maxlines;}

		if ($count > $jetzt) {
			printf(out4 "%f %s\n", ($feld[0] + $xsum), $feld[6]);
			print OUT8 "$feld[6]\n";
			if ($feld[0] > $xmax) { $xmax = $feld[0];}  
			if ($feld[6] > $ymax) { $ymax = $feld[6];}
		}
	} # end foreach

	$xsum = $xsum + $xmax;
	if ($c_tsh > 1) { print "\n\tWarning:$in_file has more than one \"Total stat het\" region\n";}
} ## end ReadINFO

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {

#	$xsum = 3700;	# for a fixed axle in every picture
	$ymax = 1;						# y-achse auf ganze HINFO-Zahl bringen
	@buff =	split('\.',$ymax);		# round to integer
	$ymax=$buff[0];
	
	open (out3,"> chr_line") || die "could not open chr_line :$!\n";

	if ($hline=1){
		for (my $i= 0.2; $i<$ymax;$i=$i+0.2) {			# horizontale Linien bei INFOs = 1,2,3,4,..
			print out3 "0 $i\n$xsum $i\n\n";
		}
	}

	foreach (@xsums){	# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print out3 "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (out3);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {

	$titel = "$pfad - GH2 INFO  - $datum";
	open (out5,"> myghinfo.gp") || die "could not open myghinfo.gp :$!\n";

	print out5 "set autoscale\n";
	print out5 "set title \"$titel\"\n" ;
	print out5 "set xlabel \"cM\"\n";
	print out5 "set ylabel \"Information Content\"\n";
	print out5 "set size 1,0.5\n";
#	my $ypos = $ymax + (($ymax-$ymin)/20);
#	foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print out5 "set terminal postscript\n";
	print out5 "set output \"gh_snp_info.ps\"\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'INFO.plt\' with lines, \'chr_line\' with lines\n";
	print out5 "set terminal x11\n";
	print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'INFO.plt\' with lines, \'chr_line\' with lines\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
					

#################################   H E L P   #################################  
sub Help {

print " * * * * * * Created by Franz Rueschendorf 2-Jan-2003  * * * * * * \n";
print " * * * * * * * * * Max-Delbrueck-Center, Berlin  * * * * * * * * * \n";
print " * * * * * * * * * email: fruesch\@mdc-berlin.de  * * * * * * * * * \n\n";
print "  Usage        : perl [path]gh_snp_info.pl [-h] [-clean] [-nodelete]\n";
print "  Infiles      : gh_<v>.out (default) \n";
print "  Outfiles     : myghinfo.gp, chr_line and INFO.plt \n" ;
print "  View results : gnuplot myghinfo.gp\n\n";
exit 0;
}

################################   C L E A N   ################################  

sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
unlink glob("chr_line");
exit 0;
}

### end end gh_snp_info.pl
